@@ -5,7 +5,7 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2022
-;; Version: 0.3
+;; Version: 0.4
 ;; Package-Requires: ((emacs "27.1"))
 ;; Homepage: https://github.com/minad/org-modern
 
@@ -34,7 +34,8 @@
 
 (require 'org)
 (eval-when-compile
-  (require 'cl-lib))
+  (require 'cl-lib)
+  (require 'subr-x))
 
 (defgroup org-modern nil
   "Modern looks for Org."
@@ -48,7 +49,7 @@
   (when (facep 'org-modern-label)
     (set-face-attribute
      'org-modern-label nil
-     :inherit (and org-modern-variable-pitch 'variable-pitch)
+     :inherit org-modern-variable-pitch
      :box (when org-modern-label-border
             (let ((border (if (eq org-modern-label-border 'auto)
                               (max 3 (cond
@@ -75,7 +76,7 @@ A value between 0.1 and 0.4 of `line-spacing' is recommended."
   :type '(choice (const nil) (const auto) integer)
   :set #'org-modern--setter)
 
-(defcustom org-modern-star ["◉""○""◈""◇""⁕"]
+(defcustom org-modern-star ["◉""○""◈""◇""✳"]
   "Replacement strings for headline stars for each level.
 Set to nil to disable styling the headlines."
   :type '(choice (const nil) (vector string)))
@@ -89,8 +90,15 @@ Set to nil to disable styling the headlines."
 
 (defcustom org-modern-timestamp t
   "Prettify time stamps, e.g. <2022-03-01>.
-Set to nil to disable styling the time stamps."
-  :type 'boolean)
+Set to nil to disable styling the time stamps. In order to use custom
+timestamps, the format should be (DATE . TIME) where DATE is the format
+for date, and TIME is the format for time. DATE and TIME must be
+surrounded with space. For the syntax, refer to `format-time-string'."
+  :type '(choice
+          (const :tag "Disable time stamp styling" nil)
+          (const :tag "Enable timestamp styling" t)
+          (const :tag "Use format YYYY-MM-DD HH:MM" (" %Y-%m-%d " . " %H:%M "))
+          (cons :tag "Custom format" string string)))
 
 (defcustom org-modern-table t
   "Prettify tables."
@@ -118,21 +126,35 @@ Set to nil to disable styling list bullets."
   :type '(alist :key-type character :value-type string))
 
 (defcustom org-modern-checkbox
-  '((?X . #("▢✓" 0 2 (composition ((2)))))
-    (?- . #("▢–" 0 2 (composition ((2)))))
-    (?\s . #("▢" 0 1 (composition ((1))))))
+  '((?X . "☑")
+    (?- . #("□–" 0 2 (composition ((2)))))
+    (?\s . "□"))
   "List of check box replacement strings.
 Set to nil to disable styling checkboxes."
   :type '(alist :key-type character :value-type string))
 
 (defcustom org-modern-horizontal-rule t
-  "Prettify horizontal rulers."
-  :type 'boolean)
+  "Prettify horizontal rulers.
+The value can either be a boolean to enable/disable style or display
+replacement expression, e.g., a string."
+  :type '(choice boolean sexp))
 
 (defcustom org-modern-todo t
   "Prettify todo keywords, see `org-todo-keywords'."
   :type 'boolean)
 
+(defcustom org-modern-todo-faces nil
+  "Faces for todo keywords.
+This is an alist, with todo keywords in the car
+and faces in the cdr. Example:
+
+  (setq org-modern-todo-faces
+    (quote ((\"TODO\" :background \"red\"
+                    :foreground \"yellow\"))))"
+  :type '(repeat
+          (cons (string :tag "Keyword")
+                (sexp   :tag "Face   "))))
+
 (defcustom org-modern-tag t
   "Prettify tags in headlines, e.g., :tag1:tag2:."
   :type 'boolean)
@@ -143,10 +165,30 @@ Set to nil to disable styling checkboxes."
 
 (defcustom org-modern-keyword t
   "Prettify keywords like #+title.
-If set to a string, e.g., \"‣\", the string is used as replacement for #+."
-  :type '(choice (boolean :tag "Hide keyword prefix")
-                 (string :tag "Custom replacement")
-                 (const :tag "Triangle bullet" "‣")))
+If set to a string, e.g., \"‣\", the string is used as replacement for #+.
+If set to an alist of keywords and strings, the associated string will be
+used as replacement for \"#+keyword:\", with t the default key."
+  :type '(choice (boolean :tag "Hide prefix")
+                 (string :tag "Replacement")
+                 (const :tag "Triangle bullet" "‣")
+                 (alist :key-type (choice (string :tag "Keyword")
+                                          (const :tag "Default" t))
+                        :value-type (choice (string :tag "Replacement")
+                                            (const :tag "Hide prefix" t)))))
+
+(defcustom org-modern-footnote (cons nil (cadr org-script-display))
+  "Prettify footnotes.
+The car corresponds to display specification for definitions, the cdr for
+references."
+  :type '(choice (const nil) (cons sexp sexp)))
+
+(defcustom org-modern-internal-target '(" ↪ " t " ")
+  "Prettify internal link targets, e.g., <<introduction>>."
+  :type '(choice (const nil) (list string boolean string)))
+
+(defcustom org-modern-radio-target '(" ⛯ " t " ")
+  "Prettify radio link targets, e.g., <<<radio>>>."
+  :type '(choice (const nil) (list string boolean string)))
 
 (defcustom org-modern-statistics t
   "Prettify todo statistics."
@@ -157,9 +199,9 @@ If set to a string, e.g., \"‣\", the string is used as replacement for #+."
 Set to nil to disable the indicator."
   :type '(choice (const nil) (vector string)))
 
-(defcustom org-modern-variable-pitch t
-  "Prefer variable pitch for modern style."
-  :type 'boolean
+(defcustom org-modern-variable-pitch 'variable-pitch
+  "Use variable pitch for modern style labels."
+  :type 'symbol
   :set #'org-modern--setter)
 
 (defgroup org-modern-faces nil
@@ -168,6 +210,11 @@ Set to nil to disable the indicator."
   :group 'org-faces
   :group 'faces)
 
+(defface org-modern-symbol nil
+  "Face used for stars, checkboxes and progress indicators.
+You can specify a font `:family'. The font families `Iosevka', `Hack' and
+`DejaVu Sans' give decent results.")
+
 (defface org-modern-label
   `((t :height 0.9 :width condensed :weight regular :underline nil))
   "Parent face for labels.")
@@ -182,6 +229,14 @@ Set to nil to disable the indicator."
     (t :foreground "white"))
   "Face used for tag labels.")
 
+(defface org-modern-internal-target
+  '((t :inherit org-modern-done))
+  "Face used for internal link targets.")
+
+(defface org-modern-radio-target
+  '((t :inherit org-modern-done))
+  "Face used for radio link targets.")
+
 (defface org-modern-done
   '((default :inherit org-modern-label)
     (((background light)) :background "gray90" :foreground "black")
@@ -189,11 +244,13 @@ Set to nil to disable the indicator."
   "Face used for done labels.")
 
 (defface org-modern-todo
+  ;; `:inverse-video' to use todo foreground as label background
   '((t :inherit (org-todo org-modern-label)
        :weight semibold :inverse-video t))
   "Face used for todo labels.")
 
 (defface org-modern-priority
+  ;; `:inverse-video' to use priority foreground as label background
   '((t :inherit (org-priority org-modern-label)
        :weight semibold :inverse-video t))
   "Face used for priority labels.")
@@ -207,10 +264,11 @@ Set to nil to disable the indicator."
   "Face used for active date labels.")
 
 (defface org-modern-time-active
+  ;; Use `:distant-foreground' to ensure readability if `hl-line-mode' is used.
   '((default :inherit org-modern-label :weight semibold)
     (((background light))
-     :background "gray35" :foreground "white")
-    (t :background "gray75" :foreground "black"))
+     :background "gray35" :foreground "white" :distant-foreground "black")
+    (t :background "gray75" :foreground "black" :distant-foreground "white"))
   "Face used for active time labels.")
 
 (defface org-modern-date-inactive
@@ -221,9 +279,10 @@ Set to nil to disable the indicator."
   "Face used for inactive date labels.")
 
 (defface org-modern-time-inactive
+  ;; Use `:distant-foreground' to ensure readability if `hl-line-mode' is used.
   '((default :inherit org-modern-label :background "gray50")
-    (((background light)) :foreground "gray95")
-    (t :foreground "gray5"))
+    (((background light)) :foreground "gray95" :distant-foreground "gray5")
+    (t :foreground "gray5" :distant-foreground "gray95"))
   "Face used for inactive time labels.")
 
 (defface org-modern-horizontal-rule
@@ -240,9 +299,23 @@ Set to nil to disable the indicator."
   (let ((beg (match-beginning 1))
         (end (match-end 1)))
     (put-text-property
-     beg end
-     'display (alist-get (char-after (1+ beg))
-                         org-modern-checkbox))))
+     beg end 'display
+     (propertize (alist-get (char-after (1+ beg))
+                            org-modern-checkbox)
+                 'face 'org-modern-symbol))))
+
+(defun org-modern--keyword ()
+  "Prettify keywords according to `org-modern-keyword'."
+  (let ((beg (match-beginning 0))
+        (end (match-end 0))
+        (rep (assoc (match-string 2) org-modern-keyword)))
+    (unless rep
+      (setq rep (assq t org-modern-keyword) end (match-end 1)))
+    (pcase (cdr rep)
+      ('t (put-text-property beg (match-end 1) 'invisible t))
+      ((pred stringp)
+       (put-text-property beg end 'display
+                          (propertize (cdr rep) 'face 'org-modern-symbol))))))
 
 (defun org-modern--statistics ()
   "Prettify headline todo statistics."
@@ -256,7 +329,9 @@ Set to nil to disable the indicator."
                          (if (= q 0)
                              1.0
                            (/ (* 1.0 (string-to-number (match-string 3))) q))))))))
-        (setq label (concat (aref org-modern-progress idx) " " label))))
+        (setq label (concat (propertize (aref org-modern-progress idx)
+                                        'face 'org-modern-symbol)
+                            " " label))))
     (setq label (concat " " label " "))
     (add-text-properties (1- (match-beginning 1)) (1+ (match-end 1))
                          `(display ,label face org-modern-statistics))))
@@ -273,17 +348,12 @@ Set to nil to disable the indicator."
       (while (search-forward ":" end 'noerror)
         (when colon
           (put-text-property
-           colon
-           (1+ colon)
-           'display (format #(" %c" 1 3 (cursor t)) (char-after colon)))
+           colon (1+ colon) 'display
+           (format #(" %c" 1 3 (cursor t)) (char-after colon)))
           (put-text-property
-           (- (point) 2)
-           (1- (point))
-           'display (format "%c " (char-before (1- (point)))))
-          (put-text-property
-           colon
-           (1- (point))
-           'face 'org-modern-tag))
+           (- (point) 2) (1- (point)) 'display
+           (format "%c " (char-before (1- (point)))))
+          (put-text-property colon (1- (point)) 'face 'org-modern-tag))
         (setq colon (point))
         (add-text-properties (1- colon) colon colon-props)))))
 
@@ -292,60 +362,65 @@ Set to nil to disable the indicator."
   (let ((todo (match-string 1))
         (beg (match-beginning 1))
         (end (match-end 1)))
+    (put-text-property beg (1+ beg) 'display
+                       (format #(" %c" 1 3 (cursor t)) (char-after beg)))
+    (put-text-property (1- end) end 'display
+                       (format "%c " (char-before end)))
     (put-text-property
-     beg (1+ beg)
-     'display (format #(" %c" 1 3 (cursor t)) (char-after beg)))
-    (put-text-property
-     (1- end) end
-     'display (format "%c " (char-before end)))
-    (put-text-property
-     beg end
-     'face
-     (if (member todo org-done-keywords)
-         'org-modern-done
-       'org-modern-todo))))
+     beg end 'face
+     (if-let (face (cdr (assoc todo org-modern-todo-faces)))
+         `(:inherit (,face org-modern-label))
+       (if (member todo org-done-keywords)
+           'org-modern-done
+         'org-modern-todo)))))
 
 (defun org-modern--timestamp ()
   "Prettify timestamps."
-  (let* ((active (eq (char-after (match-beginning 0)) ?<))
+  (let* ((beg (match-beginning 0))
+         (end (match-end 0))
+         (tbeg (match-beginning 2))
+         (tend (match-end 2))
+         (active (eq (char-after beg) ?<))
          (date-face (if active
                         'org-modern-date-active
                       'org-modern-date-inactive))
          (time-face (if active
                         'org-modern-time-active
                       'org-modern-time-inactive)))
-    (put-text-property
-     (match-beginning 0)
-     (1+ (match-beginning 0))
-     'display " ")
-    (put-text-property
-     (1- (match-end 0))
-     (match-end 0)
-     'display " ")
-    ;; year-month-day
-    (put-text-property
-     (match-beginning 0)
-     (if (eq (match-beginning 2) (match-end 2)) (match-end 0) (match-end 1))
-     'face date-face)
-    ;; hour:minute
-    (unless (eq (match-beginning 2) (match-end 2))
-      (put-text-property
-       (1- (match-end 1))
-       (match-end 1)
-       'display (format "%c " (char-before (match-end 1))))
-      (put-text-property
-       (match-beginning 2)
-       (match-end 0)
-       'face time-face))))
+    (remove-list-of-text-properties beg end '(display))
+    (if (consp org-modern-timestamp)
+        (let* ((time (save-match-data
+                       (encode-time
+                        (org-fix-decoded-time
+                         (org-parse-time-string
+                          (buffer-substring beg end))))))
+               (fmt org-modern-timestamp)
+               (date-str (format-time-string (car fmt) time))
+               (time-str (format-time-string (cdr fmt) time)))
+          ;; year-month-day
+          (add-text-properties beg (if (eq tbeg tend) end tbeg)
+                               `(face ,date-face display ,date-str))
+          ;; hour:minute
+          (unless (eq tbeg tend)
+            (add-text-properties tbeg end
+                                 `(face ,time-face display ,time-str))))
+      (put-text-property beg (1+ beg) 'display " ")
+      (put-text-property (1- end) end 'display " ")
+      ;; year-month-day
+      (put-text-property beg (if (eq tbeg tend) end tbeg) 'face date-face)
+      ;; hour:minute
+      (unless (eq tbeg tend)
+        (put-text-property (1- tbeg) tbeg
+                           'display (format "%c " (char-before tbeg)))
+        (put-text-property tbeg end 'face time-face)))))
 
 (defun org-modern--star ()
   "Prettify headline stars."
   (let ((level (- (match-end 1) (match-beginning 1))))
     (put-text-property
-     (match-beginning 2)
-     (match-end 2)
-     'display
-     (aref org-modern-star (min (1- (length org-modern-star)) level)))))
+     (match-beginning 2) (match-end 2) 'display
+     (propertize (aref org-modern-star (min (1- (length org-modern-star)) level))
+                 'face 'org-modern-symbol))))
 
 (defun org-modern--table ()
   "Prettify vertical table lines."
@@ -392,10 +467,6 @@ Set to nil to disable the indicator."
                      (put-text-property i (1+ i) 'display
                                         (if (= 0 (mod i 2)) sp1 sp2)))))))))
 
-
-(define-fringe-bitmap 'org-modern--block-inner (make-vector 1 #x80) nil nil '(top t))
-(define-fringe-bitmap 'org-modern--block-begin (vconcat (make-vector 20 0) [#xFF] (make-vector 107 #x80)) nil nil 'top)
-(define-fringe-bitmap 'org-modern--block-end (vconcat (make-vector 107 #x80) [#xFF] (make-vector 20 0)) nil nil 'bottom)
 (defun org-modern--block-fringe ()
   "Prettify blocks with fringe bitmaps."
   ;; Do not add source block fringe markers if org-indent-mode is
@@ -405,7 +476,7 @@ Set to nil to disable the indicator."
     (save-excursion
       (goto-char (match-beginning 0))
       (add-text-properties
-       (point) (min (1+ (line-end-position)) (point-max))
+       (point) (min (line-end-position) (point-max))
        '(wrap-prefix
          #(" " 0 1 (display (left-fringe org-modern--block-begin org-block-begin-line)))
          line-prefix
@@ -419,7 +490,7 @@ Set to nil to disable the indicator."
                 (re-search-forward
                  "^[ \t]*#\\+end_" (line-end-position) 'noerror)))
             (add-text-properties
-             (point) (min (1+ (line-end-position)) (point-max))
+             (point) (min (line-end-position) (point-max))
              '(wrap-prefix
                #(" " 0 1 (display (left-fringe org-modern--block-end org-block-begin-line)))
                line-prefix
@@ -442,6 +513,12 @@ Set to nil to disable the indicator."
   :group 'org-modern
   (cond
    (org-modern-mode
+    (unless (fringe-bitmap-p 'org-modern--block-inner)
+      (let* ((g (ceiling (frame-char-height) 1.8))
+             (h (- (default-line-height) g)))
+        (define-fringe-bitmap 'org-modern--block-inner [128] nil nil '(top t))
+        (define-fringe-bitmap 'org-modern--block-begin (vconcat (make-vector g 0) [#xFF] (make-vector (- 127 g) #x80)) nil nil 'top)
+        (define-fringe-bitmap 'org-modern--block-end (vconcat (make-vector (- 127 h) #x80) [#xFF] (make-vector h 0)) nil nil 'bottom)))
     (org-modern--update-label-face)
     (setq
      org-modern--keywords
@@ -458,13 +535,14 @@ Set to nil to disable the indicator."
            (2 '(face nil display " "))
            (3 '(face nil display " ")))))
       (when org-modern-todo
-        `((,(format "^\\*+ +%s " (regexp-opt org-todo-keywords-1 t)) (0 (org-modern--todo)))))
+        `((,(format "^\\*+ +%s " (regexp-opt org-todo-keywords-1 t))
+           (0 (org-modern--todo)))))
       (when org-modern-keyword
-        `(("^[ \t]*\\(#\\+\\)\\S-" 1
-           '(face nil
-                  ,@(if (stringp org-modern-keyword)
-                       `(display ,org-modern-keyword)
-                     '(invisible t))))))
+        `(("^[ \t]*\\(#\\+\\)\\([^: \t\n]+\\):"
+           ,@(pcase org-modern-keyword
+               ('t '(1 '(face nil invisible t)))
+               ((pred stringp) `(1 '(face nil display ,org-modern-keyword)))
+               (_ '(0 (org-modern--keyword)))))))
       (when org-modern-checkbox
         '(("^[ \t]*\\(?:[-+*]\\|[0-9]+[.)]\\)[ \t]+\\(\\[[ X-]\\]\\)[ \t]"
            (0 (org-modern--checkbox)))))
@@ -474,19 +552,56 @@ Set to nil to disable the indicator."
            ,@(and (eq org-modern-hide-stars 'leading) '((1 '(face nil invisible t))))
            ,@(and (eq org-modern-hide-stars t) '((0 '(face nil invisible t)))))))
       (when org-modern-horizontal-rule
-        '(("^-\\{5,\\}$" 0 '(face org-modern-horizontal-rule display (space :width text)))))
+        `(("^[ \t]*-\\{5,\\}$" 0
+           '(face org-modern-horizontal-rule display
+                  ,(if (eq org-modern-horizontal-rule t)
+                       '(space :width text)
+                     org-modern-horizontal-rule)))))
       (when org-modern-table
         '(("^[ \t]*\\(|.*|\\)[ \t]*$" (0 (org-modern--table)))))
       (when org-modern-block
-        '(("^[ \t]*#\\+\\(?:begin\\|BEGIN\\)_\\S-" (0 (org-modern--block-fringe)))
-          ("^\\([ \t]*#\\+\\(?:begin\\|BEGIN\\)_\\)\\(\\S-+\\).*"
+        '(("^[ \t]*#\\+\\(?:begin\\|BEGIN\\)_\\S-"
+           (0 (org-modern--block-fringe)))
+          ("^\\([ \t]*#\\+\\(?:begin\\|BEGIN\\)_\\)\\(\\S-+\\)"
            (1 '(face nil display (space :width (3))))
            (2 'org-modern-block-keyword append))
-          ("^\\([ \t]*#\\+\\(?:end\\|END\\)_\\)\\(\\S-+\\).*"
+          ("^\\([ \t]*#\\+\\(?:end\\|END\\)_\\)\\(\\S-+\\)"
            (1 '(face nil display (space :width (3))))
            (2 'org-modern-block-keyword append))))
       (when org-modern-tag
-        '(("^\\*+.*?\\( \\)\\(:.*:\\)[ \t]*$" (0 (org-modern--tag)))))
+        `((,(concat "^\\*+.*?\\( \\)\\(:\\(?:" org-tag-re ":\\)+\\)[ \t]*$")
+           (0 (org-modern--tag)))))
+      (when org-modern-footnote
+        `(("^\\(\\[fn:\\)[[:word:]-_]+\\]" ;; Definition
+           ,@(if-let (x (car org-modern-footnote))
+                 `((0 '(face nil display ,x))
+                   (1 '(face nil display ,(propertize "[" 'display x))))
+               '((1 '(face nil display "[")))))
+          ("[^\n]\\(\\(\\[fn:\\)[[:word:]-_]+\\]\\)" ;; Reference
+           ,@(if-let (x (cdr org-modern-footnote))
+                 `((1 '(face nil display ,x))
+                   (2 '(face nil display ,(propertize "[" 'display x))))
+               '((2 '(face nil display "[")))))))
+      (let ((target "\\([^<>\n\r\t ][^<>\n\r]*?[^<>\n\r\t @$]\\|[^<>\n\r\t @$]\\)"))
+        (append
+         (when org-modern-internal-target
+           `((,(format "\\(<<\\)%s\\(>>\\)" target)
+              (0 '(face org-modern-internal-target) t)
+              (1 '(face nil display ,(propertize (car org-modern-internal-target)
+                                                 'face 'org-modern-symbol)))
+              (3 '(face nil display ,(propertize (caddr org-modern-internal-target)
+                                                 'face 'org-modern-symbol)))
+              ,@(unless (cadr org-modern-internal-target)
+                  '((2 '(face nil invisible t)))))))
+         (when org-modern-radio-target
+           `((,(format "\\(<<<\\)%s\\(>>>\\)" target)
+              (0 '(face org-modern-radio-target) t)
+              (1 '(face nil display ,(propertize (car org-modern-radio-target)
+                                                 'face 'org-modern-symbol)))
+              (3 '(face nil display ,(propertize (caddr org-modern-radio-target)
+                                                 'face 'org-modern-symbol)))
+              ,@(unless (cadr org-modern-radio-target)
+                  '((2 '(face nil invisible t)))))))))
       (when org-modern-timestamp
         '(("\\(?:<\\|\\[\\)\\([0-9]\\{4\\}-[0-9]\\{2\\}-[0-9]\\{2\\}\\(?: [[:word:]]+\\)?\\(?: [.+-]+[0-9ymwdh/]+\\)*\\)\\(\\(?: [0-9:-]+\\)?\\(?: [.+-]+[0-9ymwdh/]+\\)*\\)\\(?:>\\|\\]\\)"
            (0 (org-modern--timestamp)))
@@ -497,10 +612,12 @@ Set to nil to disable the indicator."
         '((" \\[\\(\\([0-9]+\\)%\\|\\([0-9]+\\)/\\([0-9]+\\)\\)\\]" (0 (org-modern--statistics)))))))
     (font-lock-add-keywords nil org-modern--keywords 'append)
     (advice-add #'org-unfontify-region :after #'org-modern--unfontify))
-   (t (font-lock-remove-keywords nil org-modern--keywords)
-      (let ((org-modern-mode t))
-        (org-modern--unfontify (point-min) (point-max)))))
-  (font-lock-flush))
+   (t (font-lock-remove-keywords nil org-modern--keywords)))
+  (save-restriction
+    (widen)
+    (let ((org-modern-mode t))
+      (org-modern--unfontify (point-min) (point-max)))
+    (font-lock-flush)))
 
 (defun org-modern--unfontify (beg end &optional _)
   "Unfontify prettified elements between BEG and END."
@@ -514,5 +631,46 @@ Set to nil to disable the indicator."
            '(display face invisible)
          '(wrap-prefix line-prefix display face invisible))))))
 
+;;;###autoload
+(defun org-modern-agenda ()
+  "Finalize Org agenda highlighting."
+  (save-excursion
+    (save-match-data
+      ;; Todo keywords
+      (goto-char (point-min))
+      (let ((re (format ": +%s "
+                        (regexp-opt
+                         (append org-todo-keywords-for-agenda
+                                 org-done-keywords-for-agenda) t)))
+            (org-done-keywords org-done-keywords-for-agenda))
+        (while (re-search-forward re nil 'noerror)
+          (org-modern--todo)))
+      ;; Tags
+      (goto-char (point-min))
+      (let ((re (concat "\\( \\)\\(:\\(?:" org-tag-re ":\\)+\\)[ \t]*$")))
+        (while (re-search-forward re nil 'noerror)
+          (org-modern--tag)))
+      ;; Priorities
+      (goto-char (point-min))
+      (while (re-search-forward "\\(\\[\\)#.\\(\\]\\)" nil 'noerror)
+        ;; For some reason the org-agenda-fontify-priorities adds overlays?!
+        (when-let (ov (overlays-at (match-beginning 0))) (overlay-put (car ov) 'face nil))
+        (put-text-property (match-beginning 0) (match-end 0) 'face 'org-modern-priority)
+        (put-text-property (match-beginning 1) (match-end 1) 'display " ")
+        (put-text-property (match-beginning 2) (match-end 2) 'display " ")))))
+
+;;;###autoload
+(define-globalized-minor-mode global-org-modern-mode
+  org-modern-mode org-modern--on
+  :group 'org-modern
+  (if global-org-modern-mode
+      (add-hook 'org-agenda-finalize-hook #'org-modern-agenda)
+    (remove-hook 'org-agenda-finalize-hook #'org-modern-agenda)))
+
+(defun org-modern--on ()
+  "Enable `org-modern' in every Org buffer."
+  (when (derived-mode-p #'org-mode)
+    (org-modern-mode)))
+
 (provide 'org-modern)
 ;;; org-modern.el ends here
